SENTENCE_BERT_STS_MODEL_NAME = 'distilbert-base-nli-stsb-mean-tokens'

CIFAR_10_CLASS_NAMES = ["airplane", "automobile", "bird", "cat", "deer",
                        "dog", "frog", "horse", "ship", "truck"]
